<?php

namespace Teknisa\CADEST\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Teknisa\Libs\Util\Environment;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\Libs\Service\CRUD;

class ProdutoCrud extends \Teknisa\CADEST\Controllers\TeknisaCrud{

	const     PRODUTO_DE_ESTOQUE = 'E';
	const     SEM_REGISTRO = 0;
	private   $newCodeProd;
	protected $newCode;
	protected $environment;
	protected $dataSourceName = 'produto_produto';
	protected $dataSourceNameToSave = 'produto';
	protected $dataSourceManager;
	protected $entityManager;

	function __construct(CRUD $crudService, ParameterBag $parameterBag, $newCode, Environment $environment, $dataSourceManager, $entityManager){
		$this->dataSourceManager = $dataSourceManager;
		$this->newCode = $newCode;
		$this->environment = $environment;
		$this->entityManager = $entityManager;
		parent::__construct($crudService, $parameterBag);
	}

	public function save(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		if($row['IDESTOCOMP'] == self::PRODUTO_DE_ESTOQUE) {
			if($row['__is_new']) {
				$row['CDPRODUTO'] = $this->newCode->getCodeByTableName('PRODUTO', $this->environment->getNrOrgTrab());
				$row['CDPRODESTO'] = $row['CDPRODUTO'];
				$row['CDARVPROD'] = $row['CDPRODUTO'];
			}else{
				if($row['IDPRODATIVPR'] != 'S')
					$this->validaAondeUtilizado($row, $response);
			}
		}else{
			if($row['__is_new']) {
				$row['CDPRODUTO'] = $this->newCode->getCodeByTableName('PRODUTO', $this->environment->getNrOrgTrab());
				$row['CDARVPROD'] = $row['CDPRODUTO'];
			}else{
				if($row['IDPRODATIVPR'] != 'S')
					$this->validaAondeUtilizado($row, $response);
			}
		}
		parent::save($request, $response);
	}
	
	private function validaAondeUtilizado($row, $response){
		$cdproduto = $row['CDPRODUTO'];
		$registrosEncontrados = array();
		$registrosEncontradoEm = '';
		try {
			$sql = 'SELECT (SELECT COUNT(*) 
			                  FROM PRORECEITAM 
			                 WHERE CDPRODUTO = :CDPRODUTO 
			                   AND NRORG = :NRORG) AS RECEITA,
                           (SELECT COUNT(*) AS MENUS
                              FROM (SELECT DISTINCT CDMANCIA
                                      FROM PRODSUBSEMAM 
                                     WHERE CDPRODUTO = :CDPRODUTO 
                                       AND NRORG = :NRORG
                                     GROUP BY CDMANCIA
                                     UNION ALL
                                    SELECT DISTINCT CDMANCIA
                                      FROM PROCIASUBSEMAM 
                                     WHERE CDPRODUTO = :CDPRODUTO 
                                       AND NRORG = :NRORG)) AS MENUGRIDS,
                           (SELECT COUNT(*) AS SUBGRUPOS
                              FROM (SELECT DISTINCT NRSEQSUBGRUSEMAM
                                      FROM PRODSUBSEMAM 
                                     WHERE CDPRODUTO = :CDPRODUTO 
                                       AND NRORG = :NRORG
                                     GROUP BY NRSEQSUBGRUSEMAM)) AS SUBGRUPOS
                      FROM DUAL';
            $parametros = array(
            	             'CDPRODUTO' => $cdproduto,
            	             'NRORG' => $this->environment->getNrOrgTrab()
            	          );
        	$connection = $this->entityManager->getConnection();
			$result = $connection->fetchAll($sql, $parametros);
			if($result[0]['RECEITA'] > self::SEM_REGISTRO && $result[0]['MENUGRIDS'] > self::SEM_REGISTRO && $result[0]['SUBGRUPOS'] > self::SEM_REGISTRO){
				$mensagem = "Este produto é utilizado em: <br>";
				foreach($result[0] as $column => $valor){
					if($valor > 0){
	                    switch($column) {
	                        case 'RECEITA':
	                        	$mensagem .= $valor." Receita(s)<br>";
	                        break;
	                        case 'MENUGRIDS':
	                        	$mensagem .= $valor." Manual(ais)<br>";
	                        break;
	                        case 'SUBGRUPOS':
	                        	$mensagem .= $valor." Subgrupo(s)<br>";
	                        break;
	                    }
					}
				}
				return $response->setError(new Error($mensagem, 1));
			}
		} catch (\Exception $e) {
			return $response->setError(new Error($e->getMessage(), 1));
		}
	}
}