<?php
namespace Teknisa\CADEST\Modules\Produto\Factories;

use Doctrine\ORM\EntityManager;

class Produto {
    
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getArvprodByCdarvprod($params) {
        $sql = 'SELECT CDARVPROD 
                FROM PRODUTO
                WHERE CDARVPROD = :CDARVPROD
                  AND NRORG = :NRORG';

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['CDARVPROD'];
    }

    public function getChildren($params) {
        $sql = "SELECT CDNVPRODUTO,
                  CDPRODNIVANTE,
                  CDARVPROD,
                  CDPRODUTO,
                CASE
                    WHEN REGEXP_LIKE(SUBSTR(CDARVPROD, 1, 1), '([A-Z])')
                    THEN '99'
                    ELSE SUBSTR(CDARVPROD, 1, 1)
                END ALPHANUMERIC_ORDER
                FROM PRODUTO
                WHERE (:CDARVPROD IS NULL
                  AND CDPRODNIVANTE IS NULL)
                OR (:CDARVPROD    IS NOT NULL
                  AND CDPRODNIVANTE = :CDARVPROD)
                AND NRORG          = :NRORG
                ORDER BY ALPHANUMERIC_ORDER DESC, 
                  CDARVPROD DESC";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function getMaxLevel($params) {
        $sql = "SELECT MAX(CDNVPRODUTO) AS MAXLEVEL
                FROM PRODUTO P 
                JOIN PRODFILI PF 
                  ON PF.CDPRODUTO = P.CDPRODUTO 
                  AND PF.NRORG = P.NRORG 
                WHERE (PF.IDPRODATIV = :IDPRODATIV 
                       OR NVL(:IDPRODATIV, 'T') = 'T') 
                AND P.NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['MAXLEVEL'];
    }

    public function getProductByArvprod($params) {
        $sql = "SELECT * 
                FROM PRODUTO 
                WHERE CDARVPROD = :CDARVPROD
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }

    public function getProductByCode($params) {
        $sql = "SELECT * 
                FROM PRODUTO 
                WHERE CDPRODUTO = :CDPRODUTO
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }

    public function getProductMask($params) {
        $sql = "SELECT CDPICTPROD 
                FROM PARAMGERAL 
                WHERE NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['CDPICTPROD'];
    }

    public function getProductType($params) {
        $sql = "SELECT CASE WHEN CDPRODUTO = CDPRODESTO 
                         THEN 'E' 
                         ELSE 'C' 
                       END AS IDESTOCOMP 
                FROM PRODUTO
                WHERE CDPRODUTO = :CDPRODUTO
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['IDESTOCOMP'];
    }

    public function getTpTreeProduct($params) {
        $sql = "SELECT NRTPARVPROD,
                       NRARVOREPROD,
                       CDPRODUTO,
                       NRORG,
                       DTINCLUSAO,
                       CDOPERINCLUSAO,
                       DTULTATU,
                       NRORGINCLUSAO,
                       NRORGULTATU,
                       CDOPERULTATU,
                       IDATIVO
                FROM
                    PRODARVOREPROD 
                WHERE CDPRODUTO = :CDPRODUTO 
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getFilial($params) {
        $sql = "SELECT CDFILIAL FROM FILIAL";
 
        $query = $this->connection->prepare($sql);
        $query->execute();
        return $query->fetchAll();
    }
    
    public function deleteProdfili($cdproduto, $nrorg){
        $sql = "DELETE FROM PRODFILI
            WHERE  (CDFILIAL IN (SELECT CDFILIAL FROM FILIAL)) 
            AND  (CDPRODUTO = :P_CDPRODUTO)
            AND  (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDPRODUTO" => $cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function deleteProdfabric($cdproduto, $nrorg){
        $sql = " DELETE FROM PRODFABRIC
            WHERE (CDPRODUTO = :P_CDPRODUTO)
            AND   (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDPRODUTO"=>$cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function deleteProdprinativ($cdproduto, $nrorg){
        $sql = "DELETE FROM PRODPRINATIV
            WHERE (CDPRODUTO = :P_CDPRODUTO)
            AND   (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDPRODUTO"=>$cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function deleteProdforncomp($cdproduto, $nrorg){
        $sql = "DELETE FROM PRODFORNCOMP
            WHERE (CDPRODUTO = :P_CDPRODUTO)
            AND   (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDPRODUTO"=>$cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function getLivroMagnetico() {
        $sql = "SELECT IDPADRAOLIVRO, CDLIVROMAG
              FROM LIVROMAGNETICO
             WHERE IDPADRAOLIVRO = 'S'";

        $query = $this->connection->prepare($sql);
        $query->execute();
        return $query->fetchAll();
    }
    
    public function deleteProdRelacproditgen($cdlivromag, $cdproduto, $nrorg){
         $sql = "DELETE FROM RELACPRODITGEN
             WHERE CDLIVROMAG = :P_CDLIVROMAG
               AND CDPRODUTO = :P_CDPRODUTO
               AND   (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDLIVROMAG"=>$cdlivromag,
                "P_CDPRODUTO"=>$cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function deleteProduto($cdproduto, $nrorg){
        $sql = "DELETE FROM PRODUTO
            WHERE (CDPRODUTO = :P_CDPRODUTO)
            AND   (NRORG = :P_NRORG)";
        
        $params = array(
                "P_CDPRODUTO"=>$cdproduto,
                "P_NRORG" => $nrorg
            );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
}
